@@ -240,6 +240,7 @@ QGraphicsWidget *MicroBlog::graphicsWidget()
     m_tweetsWidget = new QGraphicsWidget(m_scrollWidget);
     m_scrollWidget->setWidget(m_tweetsWidget);
     m_tweetsLayout = new QGraphicsLinearLayout(Qt::Vertical, m_tweetsWidget);
+    m_tweetsLayout->setSpacing(0);
     m_tweetsLayout->addItem(m_headerFrame);
 
     m_layout->addItem(m_scrollWidget);
@@ -741,7 +742,7 @@ void MicroBlog::createConfigurationInterface(KConfigDialog *parent)
     configUi.setupUi(configWidget);
 
     configUi.serviceUrlCombo->addItem("https://identi.ca/api/");
-    configUi.serviceUrlCombo->addItem("https://api.twitter.com/1/");
+    configUi.serviceUrlCombo->addItem("https://twitter.com/");
     configUi.serviceUrlCombo->setEditText(m_serviceUrl);
     configUi.usernameEdit->setText(m_username);
     configUi.passwordEdit->setText(m_password);
