@@ -18,6 +18,8 @@
 
 #include "comicprovider.h"
 
+#include <QtCore/QTimer>
+
 #include <KDebug>
 #include <KIO/Job>
 #include <KIO/StoredTransferJob>
@@ -32,6 +34,10 @@ class ComicProvider::Private
               mFirstStripNumber( 1 ),
               mComicDescription( service )
         {
+            mTimer = new QTimer( parent );
+            mTimer->setSingleShot( true );
+            mTimer->setInterval( 15000 );//timeout after 15 seconds
+            connect( mTimer, SIGNAL(timeout()), mParent, SLOT(slotTimeout()) );
         }
 
         void jobDone( KJob *job )
@@ -70,10 +76,23 @@ class ComicProvider::Private
             }
         }
 
+        void slotTimeout()
+        {
+            //operation took too long, abort it
+            mParent->error( mParent );
+        }
+
+        void slotFinished()
+        {
+            //everything finished, stop the timeout timer
+            mTimer->stop();
+        }
+
         ComicProvider *mParent;
         QString mRequestedId;
         QString mRequestedComicName;
         QString mComicAuthor;
+        KUrl mImageUrl;
         bool mIsCurrent;
         bool mIsLeftToRight;
         bool mIsTopToBottom;
@@ -82,6 +101,7 @@ class ComicProvider::Private
         int mRequestedNumber;
         int mFirstStripNumber;
         KPluginInfo mComicDescription;
+        QTimer *mTimer;
         QHash< KJob*, KUrl > mRedirections;
 };
 
@@ -92,18 +112,22 @@ ComicProvider::ComicProvider( QObject *parent, const QVariantList &args )
     Q_ASSERT( args.count() >= 2 );
     const QString type = args[ 0 ].toString();
 
-    if ( type == "Date" )
+    if ( type == QLatin1String( "Date" ) )
         d->mRequestedDate = args[ 1 ].toDate();
-    else if ( type == "Number" )
+    else if ( type == QLatin1String( "Number" ) )
         d->mRequestedNumber = args[ 1 ].toInt();
-    else if ( type == "String" ) {
+    else if ( type == QLatin1String( "String" ) ) {
         d->mRequestedId = args[ 1 ].toString();
 
-        int index = d->mRequestedId.indexOf( ':' );
+        int index = d->mRequestedId.indexOf( QLatin1Char( ':' ) );
         d->mRequestedComicName = d->mRequestedId.mid( 0, index );
     }
-    else
+    else {
         Q_ASSERT( false && "Invalid type passed to comic provider" );
+    }
+
+    d->mTimer->start();
+    connect( this, SIGNAL(finished(ComicProvider*)), this, SLOT(slotFinished()) );
 }
 
 ComicProvider::~ComicProvider()
@@ -210,9 +234,23 @@ QString ComicProvider::requestedComicName() const
 
 void ComicProvider::requestPage( const KUrl &url, int id, const MetaInfos &infos )
 {
-    KIO::StoredTransferJob *job = KIO::storedGet( url, KIO::Reload, KIO::HideProgressInfo );
+    //each request restarts the timer
+    d->mTimer->start();
+
+    if (id == Image) {
+        d->mImageUrl = url;
+    }
+
+    KIO::StoredTransferJob *job;
+    if ( id == Image ) {
+        //use cached information for the image if available
+        job = KIO::storedGet( url, KIO::NoReload, KIO::HideProgressInfo );
+    } else {
+        //for webpages we always reload, making sure, that changes are recognised
+        job = KIO::storedGet( url, KIO::Reload, KIO::HideProgressInfo );
+    }
     job->setProperty( "uid", id );
-    connect( job, SIGNAL( result( KJob* ) ), this, SLOT( jobDone( KJob* ) ) );
+    connect( job, SIGNAL(result(KJob*)), this, SLOT(jobDone(KJob*)) );
 
     if ( !infos.isEmpty() ) {
         QMapIterator<QString, QString> it( infos );
@@ -225,13 +263,12 @@ void ComicProvider::requestPage( const KUrl &url, int id, const MetaInfos &infos
 
 void ComicProvider::requestRedirectedUrl( const KUrl &url, int id, const MetaInfos &infos )
 {
-
     KIO::MimetypeJob *job = KIO::mimetype( url, KIO::HideProgressInfo );
     job->setProperty( "uid", id );
     d->mRedirections[job] = url;
-    connect(job, SIGNAL( redirection( KIO::Job*, KUrl ) ), this, SLOT( slotRedirection( KIO::Job*, KUrl ) ) );
-    connect(job, SIGNAL( permanentRedirection( KIO::Job*, KUrl, KUrl ) ), this, SLOT( slotRedirection( KIO::Job*, KUrl, KUrl ) ) );
-    connect(job, SIGNAL( result(KJob* ) ), this, SLOT( slotRedirectionDone( KJob* ) ) );
+    connect(job, SIGNAL(redirection(KIO::Job*,KUrl)), this, SLOT(slotRedirection(KIO::Job*,KUrl)) );
+    connect(job, SIGNAL(permanentRedirection(KIO::Job*,KUrl,KUrl)), this, SLOT(slotRedirection(KIO::Job*,KUrl,KUrl)) );
+    connect(job, SIGNAL(result(KJob*)), this, SLOT(slotRedirectionDone(KJob*)) );
 
     if ( !infos.isEmpty() ) {
         QMapIterator<QString, QString> it( infos );
@@ -275,7 +312,7 @@ QString ComicProvider::suffixType() const
     if ( !d->mComicDescription.isValid() ) {
         return QString();
     }
-    return d->mComicDescription.property( "X-KDE-PlasmaComicProvider-SuffixType" ).toString();
+    return d->mComicDescription.property( QLatin1String( "X-KDE-PlasmaComicProvider-SuffixType" ) ).toString();
 }
 
 KPluginInfo ComicProvider::description() const
@@ -288,6 +325,11 @@ KUrl ComicProvider::shopUrl() const
     return KUrl();
 }
 
+KUrl ComicProvider::imageUrl() const
+{
+    return d->mImageUrl;
+}
+
 bool ComicProvider::isLeftToRight() const
 {
     return true;
